{% load model_tags %}

{% spaceless %}
<section class="wrapper teaser_wrapper instance_teaser_wrapper {{ model }}_teaser_wrapper {{ model }}_{{ instance.pk }}_teaser_wrapper">
  <a href="{{ instance.get_absolute_url }}">{{ instance }}</a>
</section>
{% endspaceless %}
